(in-package :cl-utilities)

(define-condition read-delimited-bounds-error (error)
  ((start :initarg :start :reader read-delimited-bounds-error-start)
   (end :initarg :end :reader read-delimited-bounds-error-end)
   (sequence :initarg :sequence :reader read-delimited-bounds-error-sequence))
  (:report (lambda (condition stream)
	     (with-slots (start end sequence) condition
	       (format stream "The bounding indices ~S and ~S are bad for a sequence of length ~S"
		       start end (length sequence)))))
  (:documentation "There's a problem with the indices START and END
for SEQUENCE. See CLHS SUBSEQ-OUT-OF-BOUNDS:IS-AN-ERROR issue."))

(defun read-delimited (sequence stream &key (start 0) end
		       (delimiter #\Newline) (test #'eql) key)
  ;; Check to make sure END is in bounds
  (when (and end (> end (length sequence)))
    (restart-case (error 'read-delimited-bounds-error
			 :start start :end end :sequence sequence)
      (continue ()
	:report "Use default instead"
	(setf end nil))))
  ;; Check to make sure START is in bounds
  (when (and start (< start 0))
    (restart-case (error 'read-delimited-bounds-error
			 :start start :end end :sequence sequence)
      (continue ()
	:report "Use default instead"
	(setf start 0))))
  (let ((key (or key #'identity))
	(end (or end (length sequence))))
    ;; START and END should be positive integers by now
    (check-type start unsigned-byte)
    (check-type end unsigned-byte)
    ;; Check to make sure that START < END
    (when (< end start)
      (restart-case (error 'read-delimited-bounds-error
			   :start start :end end :sequence sequence)
	(continue ()
	  :report "Switch start and end"
	  (rotatef start end))))
    ;; Actually do the looping
    (loop for index from start
	  for char = (read-char stream nil nil)
	  for test-result = (funcall test (funcall key char) delimiter)
	  while (and char
		     (< index end)
		     (not test-result))
	  do (setf (elt sequence index) char)
	  finally (return-from read-delimited
		    (values index test-result)))))